"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
const vscode = require("vscode");
const fs = require("fs");
const path = require("path");

/**
 * This command allows the creation of a new template directly from an existing file.
 * This command can be invoked by the Command Palette or in a folder context menu on the explorer view.
 * @export
 * @param {TemplatesManager} templatesManager
 * @param {*} args
 * @returns
 */

function CreateApl(args) {

    let fileName = "";

    // ask for filename
    let inputOptions = {
        prompt: "Create Namespace:",
        placeHolder: "<namespace name> | <initial component name (optional)> | <squid-url (optional)>",
        value: fileName
    };

    vscode.window.showInputBox(inputOptions).then(filename => {

        if (!filename || filename.trim().length == 0) return false;
        const names = filename.split('|');
        const nmspc = names[0].trim();
        var cmpnt = 'component1';
        var squid = 'www.squid.com';

        if (names[1]) cmpnt = names[1].trim();
        if (names[2]) squid = names[2].trim();

        let appPath = path.join(vscode.workspace.workspaceFolders[0].uri.fsPath, nmspc);

        //create initial files
        let imagePath = path.join(appPath, "images");
        let compPath = path.join(appPath, "components");
        let contPath = path.join(appPath, "controllers");

        let loader = path.join(appPath, "loader.js");
        let index = path.join(appPath, "index.html");

        //add default component
        let initialControlPath = path.join(compPath, cmpnt);
        let initialControlXml = path.join(initialControlPath, cmpnt + ".xml");
        let initialControlJs = path.join(initialControlPath, cmpnt + ".js");
        let initialControlCss = path.join(initialControlPath, cmpnt + ".css");

        fs.mkdir(appPath, '0755');
        fs.mkdir(imagePath, '0755');
        fs.mkdir(compPath, '0755');
        fs.mkdir(contPath, '0755');

        fs.writeFile(loader, loaderContent(nmspc, cmpnt));

        fs.mkdir(initialControlPath, '0755');
        fs.writeFile(initialControlXml, xmlContent(nmspc, cmpnt));
        fs.writeFile(initialControlJs, jsContent(nmspc, cmpnt));
        fs.writeFile(initialControlCss, cssContent(nmspc, cmpnt));
        fs.writeFile(index, indexContent(nmspc, cmpnt, squid), function () {
            vscode.workspace.openTextDocument(index).then((doc) => {
                let textEditor = vscode.window.activeTextEditor;
                if (!textEditor) {
                    vscode.window.showTextDocument(doc).then((editor) => {
                        editor.document.save();
                    });
                } else {
                    vscode.window.showTextDocument(doc, textEditor.viewColumn).then((editor) => {
                        editor.document.save();
                    });
                }
            });
        });
        
    });
}

exports.CreateApl = CreateApl;

function CreateApp(args) {

    let fileName = "";

    // ask for filename
    let inputOptions = {
        prompt: "Create Namespace:",
        placeHolder: "<namespace name> | <initial component name (optional)> | <squid-url (optional)>",
        value: fileName
    };

    vscode.window.showInputBox(inputOptions).then(filename => {

        if (!filename || filename.trim().length == 0) return false;
        const names = filename.split('|');
        const nmspc = names[0].trim();
        var cmpnt = 'component1';
        var squid = 'www.squid.com';

        if (names[1]) cmpnt = names[1].trim();
        if (names[2]) squid = names[2].trim();

        let appPath = path.join(vscode.workspace.workspaceFolders[0].uri.fsPath, nmspc);

        //create initial files
        let imagePath = path.join(appPath, "images");
        let compPath = path.join(appPath, "components");
        let contPath = path.join(appPath, "controllers");

        let loader = path.join(appPath, "loader.js");

        //add default component
        let initialControlPath = path.join(compPath, cmpnt);
        let initialControlXml = path.join(initialControlPath, cmpnt + ".xml");
        let initialControlJs = path.join(initialControlPath, cmpnt + ".js");
        let initialControlCss = path.join(initialControlPath, cmpnt + ".css");

        fs.mkdir(appPath, '0755');
        fs.mkdir(imagePath, '0755');
        fs.mkdir(compPath, '0755');
        fs.mkdir(contPath, '0755');

        fs.writeFile(loader, loaderContent(nmspc, cmpnt));

        fs.mkdir(initialControlPath, '0755');
        fs.writeFile(initialControlXml, xmlContent(nmspc, cmpnt));
        fs.writeFile(initialControlCss, cssContent(nmspc, cmpnt));
        fs.writeFile(initialControlJs, jsContent(nmspc, cmpnt), function () {
            vscode.workspace.openTextDocument(initialControlJs).then((doc) => {
                let textEditor = vscode.window.activeTextEditor;
                if (!textEditor) {
                    vscode.window.showTextDocument(doc).then((editor) => {
                        editor.document.save();
                    });
                } else {
                    vscode.window.showTextDocument(doc, textEditor.viewColumn).then((editor) => {
                        editor.document.save();
                    });
                }
            });
        });
        
    });
}

exports.CreateApp = CreateApp;

function CreateComp(args) {
    let inputOptions = {
        prompt: "Create Component:",
        value: null,
        placeHolder: "<component name> | <exisitng namespace name> | <componentfolder(folder1/folder2..) (optional)>"
    };
    vscode.window.showInputBox(inputOptions).then(function (filename) {

        if (!filename || filename.trim().length == 0) return false;
        const names = filename.split('|');
        const cmpnt = names[0].trim();
        const nmspc = names[1].trim();
        var cmpntfolder = ['components'];

        if (names[2]) cmpntfolder = names[2].split('\\');

        let rootPath = vscode.workspace.workspaceFolders[0].uri.fsPath;
        let nmspcPath = path.join(rootPath, nmspc);

        if (!fs.existsSync(nmspcPath)){
            vscode.window.showInformationMessage('Namespace not exist');
            return false;
        }

        let loadrPath = path.join(nmspcPath, "loader.js");
        let jscontent = fs.readFileSync(loadrPath).toString();

        //get the components
        let rgx = new RegExp(/(?=loadComponents\()[\s\S]*?(?=\)\;)/, 'g')
        let obj = JSON.parse(jscontent.match(rgx)[0].replace('loadComponents(', ''));

        var json = 1; var ptr = nmspcPath;
        for (let folder of cmpntfolder){

            folder = folder.trim();
            if(json != 1) {
                if(!json[folder]) json[folder] = {};
                json = json[folder];
            } else {
                if(!obj[folder]) obj[folder] = {};
                json = obj[folder];
            }

            ptr = path.join(ptr, folder);
            if (!fs.existsSync(ptr)){
                fs.mkdirSync(ptr, '0755');
            }

        }
        
        json[cmpnt] = [];
        let controlPath = path.join(ptr, cmpnt);
        console.log(controlPath);
        if (!fs.existsSync(controlPath)){
            fs.mkdirSync(controlPath, '0755');
        }

        let content = JSON.stringify(obj, null, "\t");
        fs.writeFile(loadrPath, addLoaderContent(nmspc, content));
        fs.writeFile(path.join(controlPath, cmpnt + ".xml"), xmlContent(nmspc, cmpnt));
        fs.writeFile(path.join(controlPath, cmpnt + ".js"), jsContent(nmspc, cmpnt));
        fs.writeFile(path.join(controlPath, cmpnt + ".css"), cssContent(nmspc, cmpnt), 
            function () {
                vscode.workspace.openTextDocument(path.join(controlPath, cmpnt + ".js")).then((doc) => {
                    let textEditor = vscode.window.activeTextEditor;
                    if (!textEditor) {
                        vscode.window.showTextDocument(doc).then((editor) => {
                            editor.document.save();
                        });
                    } else {
                        vscode.window.showTextDocument(doc, textEditor.viewColumn).then((editor) => {
                            editor.document.save();
                        });
                    }
            });
        });
    });
}

exports.CreateComp = CreateComp;

function CreateCont(args) {
    let inputOptions = {
        prompt: "Create Controller:",
        value: null,
        placeHolder: "<controller name> | <existing namespace name> | <controllerfolder(folder1/folder2..) (optional)>"
    };
    vscode.window.showInputBox(inputOptions).then(function (filename) {

        if (!filename || filename.trim().length == 0) return false;
        const names = filename.split('|');
        const cmpnt = names[0].trim();
        const nmspc = names[1].trim();
        var cmpntfolder = ['controllers'];

        if (names[2]) cmpntfolder = names[2].split('\\');

        let rootPath = vscode.workspace.workspaceFolders[0].uri.fsPath;
        let nmspcPath = path.join(rootPath, nmspc);
        
        if (!fs.existsSync(nmspcPath)){
            vscode.window.showInformationMessage('Namespace not exist');
            return false;
        }

        let loadrPath = path.join(nmspcPath, "loader.js");
        let jscontent = fs.readFileSync(loadrPath).toString();

        //get the components
        let rgx = new RegExp(/(?=loadComponents\()[\s\S]*?(?=\)\;)/, 'g')
        let obj = JSON.parse(jscontent.match(rgx)[0].replace('loadComponents(', ''));

        var json = 1; var ptr = nmspcPath;
        for (let folder of cmpntfolder){

            folder = folder.trim();
            if(json != 1) {
                if(!json[folder]) json[folder] = {};
                json = json[folder];
            } else {
                if(!obj[folder]) obj[folder] = {};
                json = obj[folder];
            }

            ptr = path.join(ptr, folder);
            if (!fs.existsSync(ptr)){
                fs.mkdirSync(ptr, '0755');
            }

        }
        
        json[cmpnt] = [];
        let controlPath = path.join(ptr, cmpnt);
        console.log(controlPath);
        if (!fs.existsSync(controlPath)){
            fs.mkdirSync(controlPath, '0755');
        }

        let content = JSON.stringify(obj, null, "\t");
        fs.writeFile(loadrPath, addLoaderContent(nmspc, content));
        fs.writeFile(path.join(controlPath, cmpnt + ".js"), jsContent(nmspc, cmpnt, true), 
            function () {
                vscode.workspace.openTextDocument(path.join(controlPath, cmpnt + ".js")).then((doc) => {
                    let textEditor = vscode.window.activeTextEditor;
                    if (!textEditor) {
                        vscode.window.showTextDocument(doc).then((editor) => {
                            editor.document.save();
                        });
                    } else {
                        vscode.window.showTextDocument(doc, textEditor.viewColumn).then((editor) => {
                            editor.document.save();
                        });
                    }
            });
        });
    });
}

exports.CreateCont = CreateCont;

function loaderContent(nmspc, cmpnt) {
    var url = vscode.extensions.getExtension('sqreem.squidv3extension').extensionPath;
    var ldr = path.join(url, 'defaults', 'loader.js');
    var cnt = fs.readFileSync(ldr).toString();
    return eval(cnt);
}

function indexContent(nmspc, cmpnt, squrl) {
    var url = vscode.extensions.getExtension('sqreem.squidv3extension').extensionPath;
    var ldr = path.join(url, 'defaults', 'index.js');
    var cnt = fs.readFileSync(ldr).toString();
    return eval(cnt);
}

function xmlContent(nmspc, cmpnt) {
    var url = vscode.extensions.getExtension('sqreem.squidv3extension').extensionPath;
    var ldr = path.join(url, 'defaults', 'xmlns.js');
    var cnt = fs.readFileSync(ldr).toString();
    return eval(cnt);
}

function jsContent(nmspc, cmpnt, iscont) {
    
    var script = 'script.js';
    var upcmpnt = capitalize(cmpnt);

    if (iscont) script = 'cntrler.js';
    var url = vscode.extensions.getExtension('sqreem.squidv3extension').extensionPath;
    var ldr = path.join(url, 'defaults', script);
    var cnt = fs.readFileSync(ldr).toString();
    return eval(cnt);

}

function cssContent(nmspc, cmpnt) {
    var url = vscode.extensions.getExtension('sqreem.squidv3extension').extensionPath;
    var ldr = path.join(url, 'defaults', 'styles.js');
    var cnt = fs.readFileSync(ldr).toString();
    return eval(cnt);
}

function capitalize(string) {
    return string.charAt(0).toUpperCase() + string.slice(1);
}

function addLoaderContent(nmspc, json) {
    var url = vscode.extensions.getExtension('sqreem.squidv3extension').extensionPath;
    var ldr = path.join(url, 'defaults', 'ypdter.js');
    var cnt = fs.readFileSync(ldr).toString();
    return eval(cnt);
}